/***************************************************************************
                          ann_clock.h  -  description
                             -------------------
    begin                : pi kwi 18 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_CLOCK_H
#define __ANN_CLOCK_H

#include <time.h>

namespace ANN
{

  class Clock;
  class UnixClock;
  
  inline time_t  get_NAN(time_t *t) { return -1; }
  inline time_t &set_NAN(time_t &t) { return t=(time_t)(-1); }

  //             //
 // class Clock //
//             //

  class Clock
  {
  public:
    class Buffer
    {
    protected:  
      size_t  length;
      char   *buffer;
    public:  
      Buffer(size_t len=0)
      : length(len), buffer(len ? new char[len+1] : 0) {}
      Buffer(const Buffer &that)
      : length(that.length),
        buffer(that.length ? new char[that.length+1] : 0) {}
      ~Buffer() { if (buffer) delete buffer; }
      char *c_str() { return buffer; }
      const char *c_str() const { return buffer; }
      void resize(size_t len)
      {
        if (len>length)
        {
          if (buffer) delete buffer;
          buffer=new char[(length=len)+1];
        }
      }
      Buffer &operator = (const Buffer &that);
    };
    
    static Clock clock;
    
    Clock() {}
    virtual ~Clock();

    virtual size_t stamp(Buffer &buff);
    virtual time_t parse(const Buffer &buff);
  };

  //                 //
 // class UnixClock //
//                 //

  class UnixClock : public virtual Clock
  {
  public:
    static UnixClock clock;

    UnixClock() {}
    virtual ~UnixClock();

    virtual size_t stamp(Buffer &buff);
    virtual time_t parse(const Buffer &buff);
  };

} // namespace ANN

#endif /* __ANN_CLOCK_H */
