/***************************************************************************
                          ann_element.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_ELEMENT_H
#define __ANN_ELEMENT_H

#include <ctype.h>

#include <ann_loc.h>
#include <ann_term.h>
#include <ann_ds.h>
#include <ann_factory.h>

namespace ANN
{
  class Element;
  class Evaluator;
}

#include <ann_weight.h>
#include <ann_trainer.h>

namespace ANN
{

  //               //
 // class Element //
//               //

  class Element
  {
  public:
    typedef void        Common_t;
    typedef const Init *Instance_t;
    typedef Element     Base_t;

  private:
    Element();
    Element(const Element &that);

  protected:
    const char        *label;
    Size               in_size;
    static const Size  single_out_size;

    Element(Instance_t inst)
    : label(inst->dup_label()), in_size(*inst->size) {}

  public:
    virtual ~Element();

    const Size              &get_in_sizes() const { return in_size; }
    const size_t             get_in_size() const { return in_size.total_size(); }
    virtual const Size      &get_out_sizes() const;
    virtual const size_t     get_out_size() const;
    virtual const Evaluator *get_evaluator() const=0;
    virtual void             calc(const Term &in, Term &out)=0;

    /* Low level adaptation routines */
    virtual void             reset(Trainer &t, bool b_reload=false)=0;
    /* element moves its state to the very begining of adaptation */
    virtual void             prepare(Trainer &t);
    /* called every time epoch starts */
    virtual void             update(Trainer &t);
    /* called every time epoch ends */
    virtual void             adapt(const Term &in, const Term &out,
                                   const Term *out_err, Term *in_err)=0;
    /* called after each presentation, should backpropagate errors (if needed)
     * and calculate weights' partial changes/gradients */
  };

  typedef FactoryBase<Element,const Init *> NeuralFactoryBase;
  
  //                 //
 // class Evaluator //
//                 //

  class Evaluator
  {
  public:
    virtual ~Evaluator();

    virtual bool   trivial_match() const;
    /* true iff it is sufficient to compare total sizes */
    virtual bool   matching(const Size &out_size, const Size &des_size) const;
    /* compares sizes of output and desired */
    virtual double evaluate_ds(DS &ds, const Term &out, Term &err) const;
    virtual double evaluate(const Term &in, const Term &desired,
                            const Term &out, Term &err) const=0;

    double operator () (DS &ds, const Term &out, Term &err) const
    { return evaluate_ds(ds,out,err); }
    double operator () (const Term &in, const Term &desired,
                        const Term &out, Term &err) const
    { return evaluate(in,desired,out,err); }
  };

} //namespace ANN

#endif /* __ANN_ELEMENT_H */
