/***************************************************************************
                          ann_grad_weight.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_GRAD_WEIGHT_H
#define __ANN_GRAD_WEIGHT_H

#include <ctype.h>

#include <ann_weight.h>
#include <ann_wgroup_set.h>
#include <ann_wcommon.h>
#include <ann_wmember.h>

namespace ANN
{

  class GradWeight_Cmn;
  class GradWeight_Grp;
  class GradWeight;
  class MomentumWeight_Cmn;
  class MomentumWeight;

  //                      //
 // class GradWeight_Cmn //
//                      //

  class GradWeight_Cmn : public WGScommon
  {
  public:  
    double etha;
    
    GradWeight_Cmn(Log &log_, double etha_=.01)
    : WGScommon(log_), etha(etha_) {}
    GradWeight_Cmn(const GradWeight_Cmn &that)
    : WGScommon(that), etha(that.etha) {}    
  };
  
  //                      //
 // class GradWeight_Grp //
//                      //

  class GradWeight_Grp : public Wgroup_set<GradWeight,Weight::Instance_t>
  {
  private:
    GradWeight_Grp();
    GradWeight_Grp(const GradWeight_Grp &);
    
  public:
    typedef GradWeight_Cmn          Common_t;
    typedef Weight::Instance_t      Instance_t;
    typedef GradWeight_Grp          Base_t;
    typedef GradWeight              WBase_t;
    typedef Factory<GradWeight_Grp> Factory_t;
    
    double etha;
    
    GradWeight_Grp(GradWeight_Cmn &c, Instance_t inst)
    : Wgroup_set<GradWeight,Weight::Instance_t>(c,inst), etha(c.etha) {}
    
    virtual double        get_random();
  };

  //                  //
 // class GradWeight //
//                  //

  class GradWeight : public Weight
  {
  private:
    GradWeight();
    GradWeight(const GradWeight &);

  protected:
    Term d;

  public:
    typedef Wcommon<GradWeight_Grp>       Common_t;
    typedef Weight::Instance_t            Instance_t;
    typedef Weight                        Base_t;
    typedef GradWeight_Grp                WGBase_t;

    GradWeight(Common_t &c, Instance_t inst) : Weight(inst), d(*inst->size) {}
    virtual ~GradWeight();

    virtual void            prepare(Trainer &t);
    virtual void            adapt(const Term &d_);
    virtual void            update(Trainer &t);
    virtual GradWeight_Grp &the_group() const=0;
  };

  typedef Factory<Wmember<GradWeight> > GradWeight_Fact;
  
  //                          //
 // class MomentumWeight_Cmn //
//                          //

  class MomentumWeight_Cmn : public GradWeight_Cmn
  {
  public:
    double alpha;
    
    MomentumWeight_Cmn(Log &log_, double etha_=.01, double alpha_=.9)
    : GradWeight_Cmn(log_, etha_), alpha(alpha_) {}
    MomentumWeight_Cmn(const MomentumWeight_Cmn &that)
    : GradWeight_Cmn(that), alpha(that.alpha) {}
  };

  //                          //
 // class MomentumWeight_Grp //
//                          //

  class MomentumWeight_Grp
  : public Wgroup_set<MomentumWeight,Weight::Instance_t>
  {
  private:
    MomentumWeight_Grp();
    MomentumWeight_Grp(const MomentumWeight_Grp &);

  public:
    typedef MomentumWeight_Cmn          Common_t;
    typedef Weight::Instance_t          Instance_t;
    typedef MomentumWeight_Grp          Base_t;
    typedef MomentumWeight              WBase_t;
    typedef Factory<MomentumWeight_Grp> Factory_t;

    double etha;
    double alpha;

    MomentumWeight_Grp(MomentumWeight_Cmn &c, Instance_t inst)
    : Wgroup_set<MomentumWeight,Weight::Instance_t>(c,inst),
      etha(c.etha), alpha(c.alpha) {}

    virtual double        get_random();
  };


  //                      //
 // class MomentumWeight //
//                      //

  class MomentumWeight : public Weight
  {
  private:
    MomentumWeight();
    MomentumWeight(const MomentumWeight &);

  protected:
    Term d;
    Term delta;

  public:
    typedef Wcommon<MomentumWeight_Grp>       Common_t;
    typedef Weight::Instance_t                Instance_t;
    typedef Weight                            Base_t;
    typedef MomentumWeight_Grp                WGBase_t;

    MomentumWeight(Common_t &c, Instance_t inst)
    : Weight(inst), d(*inst->size), delta(*inst->size) {}
    virtual ~MomentumWeight();

    virtual void                reset(Trainer &t);
    virtual void                prepare(Trainer &t);
    virtual void                adapt(const Term &d_);
    virtual void                update(Trainer &t);
    virtual MomentumWeight_Grp &the_group() const=0;
  };

  typedef Factory<Wmember<MomentumWeight> > MomentumWeight_Fact;

} // namespace ANN

#endif /* __ANN_WEIGHT_H */
