/***************************************************************************
                          ann_log.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_LOG_H
#define __ANN_LOG_H

#include <ctype.h>

#include <ann_term.h>
#include <ann_clock.h>

namespace ANN
{

  class Log;
  class LogRec;
  class Lstore;

  //           //
 // class Log //
//           //

  class Log
  {
  protected:
    Lstore *children;
    
    virtual void new_line(int ch_marg, const char *label)=0;
    virtual void log_chr(char chr)=0;
    virtual void log_str(const char *str)=0;
    virtual void log_int(int i)=0;
    virtual void log_uint(unsigned int u)=0;
    virtual void log_double(double d)=0;
    virtual void log_term(const Term &t, const char *label);

  public:
    Log() : children(0) {}
    virtual ~Log();

    virtual Clock  &get_clock() const=0;
    virtual char    get_separator() const=0;
    virtual char    get_sublevel_beg() const=0;
    virtual char    get_sublevel_end() const=0;
    virtual size_t  get_margin() const=0;
    virtual bool    attach(LogRec &child);
    virtual bool    detach(LogRec &child);
    virtual bool    is_active() const;
    /* tells whether logging is possible */
    virtual void    reset();
    /* restarts logging */
    virtual void    prepare(size_t epoch, bool active_=true);
    /* caled whenever epoch starts */
    virtual void    stepped(size_t num, double error, bool active_=true);
    /* caled whenever step ends */
    virtual void    update(size_t epoch, double error, bool active_=true);
    /* caled whenever epoch ends */
    virtual Log    &flush()=0;
    /* synchronizes buffers with storage */

    Log &nl(int ch_marg=0, const char *label=0)
    { new_line(ch_marg, label); return *this; }
    Log &nl(const char *label) { new_line(0, label); return *this; }
    Log &log(char chr) { log_chr(chr); return *this; }
    Log &log(const char *str) { log_str(str); return *this; }
    Log &log(int i) { log_int(i); return *this; }
    Log &log(unsigned int u) { log_uint(u); return *this; }
    Log &log(double d) { log_double(d); return *this; }
    Log &log(const Term &t, const char *label=0)
    { log_term(t,label); return *this; }

    Log &operator << (char chr) { return log(chr); }
    Log &operator << (const char *str) { return log(str); }
    Log &operator << (int i) { return log(i); }
    Log &operator << (unsigned int u) { return log(u); }
    Log &operator << (double d) { return log(d); }
    Log &operator << (const Term &t) { return log(t); }
  };

  //              //
 // class LogRec //
//              //

  class LogRec : public Log
  {
  private:
    LogRec();

  protected:
    Log    *orig;
    bool    active;

    virtual void new_line(int ch_marg, const char *label);
    virtual void log_chr(char chr);
    virtual void log_str(const char *str);
    virtual void log_int(int i);
    virtual void log_uint(unsigned int u);
    virtual void log_double(double d);
    virtual void log_term(const Term &t, const char *label);
    virtual void prop_reset();
    virtual void prop_prepare(size_t epoch);
    virtual void prop_stepped(size_t num, double error);
    virtual void prop_update(size_t epoch, double error);

  public:
    LogRec(Log &orig_) : orig(&orig_), active(false) { orig->attach(*this); }
    virtual ~LogRec();

    virtual Clock  &get_clock() const;
    virtual char    get_separator() const;
    virtual char    get_sublevel_beg() const;
    virtual char    get_sublevel_end() const;
    virtual size_t  get_margin() const;
    virtual bool    is_active() const;
    virtual void    reset();
    virtual void    prepare(size_t epoch, bool active_=true);
    virtual void    stepped(size_t num, double error, bool active_=true);
    virtual void    update(size_t epoch, double error, bool active_=true);
    virtual Log    &flush();
    
    friend class Lstore;  
  };

  //              //
 // class Lstore //
//              //

  class Lstore
  {
  protected:
    Lstore() {}

    void prop_reset(LogRec &log) { log.prop_reset(); }
    void prop_prepare(LogRec &log, size_t epoch) { log.prop_prepare(epoch); }
    void prop_stepped(LogRec &log, size_t num, double error)
    { log.prop_stepped(num,error); }
    void prop_update(LogRec &log, size_t epoch, double error)
    { log.prop_update(epoch,error); }

  public:
    virtual ~Lstore();

    virtual bool  attach(LogRec &child)=0;
    virtual bool  detach(LogRec &child)=0;
    virtual void  reset()=0;
    virtual void  prepare(size_t epoch)=0;
    virtual void  stepped(size_t num, double error)=0;
    virtual void  update(size_t epoch, double error)=0;
  };

} // namespace ANN

#endif /* __ANN_LOG_H */
