/***************************************************************************
                          ann_multilayer.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_MULTILAYER_H
#define __ANN_MULTILAYER_H

#include <vector>

#include <ann_element.h>
#include <ann_factory.h>

namespace ANN
{

  //                  //
 // class Multilayer //
//                  //

  class Multilayer : public Element
  {
  protected:
    struct LayerData
    {
    public:
      Element *elem;
      Term    *in;
      Term    *err;

      LayerData(Element &elem_, Element *former);
      ~LayerData()
      { if (elem) delete elem; if (in) delete in; if (err) delete err; }
    };
    typedef std::vector<LayerData *>  Layers;
    typedef Layers::iterator          L_it;
    typedef Layers::reverse_iterator  L_rit;
    typedef NeuralFactoryBase        *Common_t;
    typedef const Init               *Instance_t;
    typedef Element                   Base_t;

    Layers layers;

    Multilayer(Instance_t inst) : Element(inst) {}

    bool add_layer(Element *elem_);
    bool add_layer(Element &elem_) { return add_layer(&elem_); }
    bool add_layer(NeuralFactoryBase *factory);
    bool add_layer(NeuralFactoryBase &factory) { return add_layer(&factory); }

    template<class Iterator>
    bool build(Iterator begin, Iterator end)
    { 
      for (Iterator it=begin; it!=end; ++it) if (!add_layer(*it)) return false; 
      return true; 
    }
    bool build(NeuralFactoryBase **arr, size_t num)
    { 
      for (size_t i=0; i<num; ++i) if (!add_layer(arr[i])) return false;
      return true;
    }
    bool build(NeuralFactoryBase **arr)
    {
      while (*arr) if (add_layer(*arr)) ++arr; else return false; 
      return true;
    }
    bool build(NeuralFactoryBase *&arr)
    {
      return build(&arr);
    }

  public:
    template<class Iterator>
    Multilayer(Instance_t inst, Iterator begin, Iterator end)
    : Element(inst) { build(begin,end); }
    Multilayer(Instance_t inst, NeuralFactoryBase **factory, size_t num)
    : Element(inst) { build(factory,num); }
    Multilayer(Instance_t inst, NeuralFactoryBase **factory)
    : Element(inst) { build(factory); }
    Multilayer(Instance_t inst, NeuralFactoryBase *&factory)
    : Element(inst) { build(factory); }

    virtual ~Multilayer();

    virtual const Size      &get_out_sizes() const;
    virtual const Evaluator *get_evaluator() const;
    virtual void             calc(const Term &in, Term &out);
    virtual void             reset(Trainer &t, bool b_reload=false);
    virtual void             prepare(Trainer &t);
    virtual void             update(Trainer &t);
    virtual void             adapt(const Term &in, const Term &out,
                                   const Term *out_err, Term *in_err);
                                   
    template<class Iterator>
    static void drop_common(Iterator begin, Iterator end)
    { while(begin!=end) delete *begin++; }
    static void drop_common(NeuralFactoryBase **arr, size_t num)
    { for (size_t i=0; i<num; ++i) delete arr[i]; }
    static void drop_common(NeuralFactoryBase **arr)
    { while (*arr) delete *arr++; }
    static void drop_common(NeuralFactoryBase *&arr)
    { drop_common(&arr); }
  };

  typedef Factory<Multilayer> MultilayerFactory;
  
} // namespace ANN

#endif /* __ANN_MULTILAYER_H */
