/***************************************************************************
                          ann_perceptron.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_PERCEPTRON_H
#define __ANN_PERCEPTRON_H

#include <ann_factory.h>
#include <ann_element.h>
#include <ann_weight.h>

namespace ANN
{

  //                  //
 // class Perceptron //
//                  //

  class Perceptron : public Element
  {
  public:
    class Activation
    {
    public:
      virtual ~Activation() {}
      virtual double fun(double x) const=0;
      virtual double deriv(double x, double y) const=0;
      double operator() (double x) const { return fun(x); }
      double operator() (double x, double y) const { return deriv(x,y); }
    };
    static class Sigmoidal : public Activation
    {
    public:
      virtual double fun(double x) const;
      virtual double deriv(double x, double y) const;
    } sigmoidal;
    static class Linear : public Activation
    {
    public:
      virtual double fun(double x) const;
      virtual double deriv(double x, double y) const;
    } linear;
    static class Evaluator : public ::ANN::Evaluator
    {
      virtual double evaluate(const Term &in, const Term &desired,
                              const Term &out, Term &err) const;
    } evaluator;
    struct Param
    {
      WeightFactoryBase *wf;
      Activation        *act;
      Size               out_size;
      bool               bias;
      Param(const Param &that)
      : wf(that.wf), act(that.act), out_size(that.out_size), bias(that.bias) {}
      Param(WeightFactoryBase *wf_, Activation *act_, bool bias_,
            const Size &out_size_)
      : wf(wf_), act(act_), out_size(out_size_), bias(bias_) {}
      Param(WeightFactoryBase *wf_, Activation *act_, bool bias_,
            const Loc &out_size_)
      : wf(wf_), act(act_), out_size(out_size_), bias(bias_) {}
      Param(WeightFactoryBase *wf_, Activation *act_, bool bias_,
            size_t dimension_, size_t *positions_=0)
      : wf(wf_), act(act_), out_size(dimension_,positions_), bias(bias_) {}
      Param(WeightFactoryBase *wf_, Activation *act_, bool bias_,
            size_t *positions_)
      : wf(wf_), act(act_), out_size(positions_), bias(bias_) {}
    };
    typedef Param                       Common_t;
    typedef const Init                 *Instance_t;
    typedef Element                     Base_t;

  private:
    Perceptron();
    Perceptron(const Perceptron &);

  protected:
    Size         out_size;
    bool         bias;
    Warray       w;
    Activation  *act;
    Term         sum;

  public:
    Perceptron(const Param &param, const Init *inst);
    virtual ~Perceptron();

    virtual const Size             &get_out_sizes() const;
    virtual const ::ANN::Evaluator *get_evaluator() const;
    virtual void                    reset(Trainer &t, bool b_reload=false);
    virtual void                    calc(const Term &in, Term &out);
    virtual void                    adapt(const Term &in, const Term &out,
                                          const Term *out_err, Term *in_err);
  };
  
  typedef Factory<Perceptron> PerceptronFactory;

} //namespace ANN

#endif /* __ANN_PERCEPTRON_H */

