/***************************************************************************
                          ann_trainer.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_TRAINER_H
#define __ANN_TRAINER_H

#include <ctype.h>

#include <ann_log.h>

namespace ANN
{
  class Trainer;
}

#include <ann_ds.h>
#include <ann_element.h>
#include <ann_weight.h>

namespace ANN
{

  //               //
 // class Trainer //
//               //

  class Trainer
  {
  private:
    Trainer(const Trainer &that);

  protected:
    Wstore  *ws;
    Log     *log;
    DS      *ds;
    Element *elem;
    double   error;
    size_t   epoch;

    Trainer(Log *log_=0, Wstore *ws_=0);
    Trainer(Element &elem_, Log *log_=0, Wstore *ws_=0);
    Trainer(DS &ds_, Element &elem_, Log *log_=0, Wstore *ws_=0);

    virtual bool matching(const DS &ds_, const Element &elem_) const;

    virtual void prepare();
    virtual void update();
    
  public:
    Trainer(DS &ds_, Log *log_=0, Wstore *ws_=0);

    virtual ~Trainer();

    double          get_error() const { return error; }
    size_t          get_epoch() const { return epoch; }
    Log            *get_log() const { return log; }
    DS             *get_ds() const { return ds; }
    virtual bool    act_on(DS &ds_);
    virtual bool    act_on(Element &elem_);
    virtual void    reset(bool b_reload=false);
    virtual void    randomize();
    void            register_weight(Weight &w);
    virtual bool    load(Parser_Log &parser, time_t begin=0,
                         time_t end=(time_t)-1);  
    virtual void    store(Log &log_); 
    virtual size_t  train(size_t epochs=1, double stop_error=0,
                          size_t chk_last=1);
  };

} //namespace ANN

#endif /* __ANN_TRAINER_H */
