/***************************************************************************
                          ann_weight.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_WEIGHT_H
#define __ANN_WEIGHT_H

#include <ctype.h>
#include <time.h>

namespace ANN
{

  class  Weight;
  class  Wgroup;
  class  Wstore;
  
} // namespace ANN

#include <ann_init.h>
#include <ann_term.h>
#include <ann_log.h>
#include <ann_parser_log.h>
#include <ann_trainer.h>
#include <ann_factory.h>

namespace ANN
{

  //              //
 // class Weight //
//              //

  class Weight
  {
  private:
    Weight();
    Weight(const Weight &that);
    
  protected:
    char *label;
    Term  w;

  public:
    typedef void        Common_t;
    typedef const Init *Instance_t;
    typedef Weight      Base_t;

    Weight(const Init *inst)
    : label(inst->dup_label()), w(*inst->size) {}
    virtual ~Weight();

    const Term     &values() const { return w; }
    const Size     &get_sizes() const { return w.get_sizes(); }
    const size_t    get_size() const { return w.get_size(); }
    const char     *get_label() const { return label; }
    virtual bool    has_label(const char *label_);
    virtual Wgroup *get_group() const;
    virtual void    reset(Trainer &t);    /* resets weight before training */
    virtual void    prepare(Trainer &t);  /* prepares to an epoch */
    virtual void    adapt(const Term &d_)=0;    /* called during training */
    virtual void    update(Trainer &t)=0; /* updates value */
    virtual void    randomize();          /* randomizes value */
    virtual bool    load(Parser_Log &parser, time_t begin=0,
                         time_t end=(time_t)-1); /* loads from a log */
    virtual void    store(Log &log);             /* stores to a log */
  };

  typedef FactoryBase<Weight,const Init *> WeightFactoryBase;
  typedef Array<Weight *>                  Warray;
  
  //              //
 // class Wgroup //
//              //

  class Wgroup
  {
  private:
    Wgroup(Wgroup &that);

  protected:
    Wgroup() {}

    virtual bool signature_find(Parser_Log &parser, time_t begin=0,
                                time_t end=(time_t)-1);
    virtual void signature_write(Log &log, Weight &w);
    
  public:
    virtual ~Wgroup();

    virtual bool   has_label(const char *label_)=0;
    virtual void   reset(Trainer &trainer);
    virtual void   prepare(Trainer &trainer);
    virtual void   prepared(Trainer &trainer);
    virtual void   update(Trainer &trainer);
    virtual void   updated(Trainer &trainer);
    virtual double get_random();
    static double  def_random();
    virtual bool   load(Parser_Log &parser, time_t begin=0,
                        time_t end=(time_t)-1); /* loads from a log */
    virtual void   store(Log &log);             /* stores to a log */
  };

  //              //
 // class Wstore //
//              //

  class Wstore
  {
  private:
    Wstore(const Wstore &that);

  protected:
    Wstore() {}

  public:
    virtual ~Wstore();

    virtual void register_weight(Weight &w)=0;
    virtual void erase_weights()=0;
    virtual void reset(Trainer &t)=0;           /* resets weights before
                                                 * training */
    virtual void prepare(Trainer &t)=0;         /* prepares to an epoch */
    virtual void update(Trainer &t)=0;          /* updates value */
    virtual void randomize()=0;                 /* randomizes value */
    virtual bool load(Parser_Log &parser, time_t begin=0,
                      time_t end=(time_t)-1)=0; /* loads from a log */
    virtual void store(Log &log)=0;             /* stores to a log */
  };

} // namespace ANN

#endif /* __ANN_WEIGHT_H */
