/***************************************************************************
                          ann_wfactory.h  -  description
                             -------------------
    begin                : pon kwi 21 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_WFACTORY_H
#define __ANN_WFACTORY_H

#include <ann_weight.h>
#include <factory.h>

namespace ANN
{

  template <class WG, class Instance=typename WG::Instance_t>
  class Winstance;
  template <class W, class WG, class WCommon=typename W::Common_t,
            class WGCommon=typename WG::Common_t,
            class Instance=typename WG::Instance_t>
  class WFactory;

  template<class WG, class Instance>
  inline void wfactory_can_handle(WG &wg, Instance inst) { return true; }
  template<class WG>
  inline void wfactory_can_handle(WG &wg) { return true; }
  template<class WG, class W, class Instance>
  inline void wfactory_attach(WG &wg, W &w, Instance inst) {}
  template<class WG, class W>
  inline void wfactory_attach(WG &wg, W &w) {}
    
  //template <class Group, class Instance>//
 // class Winstance                      //
//                                      //

  template <class WG, class Instance>
  class Winstance
  {
  public:
    typedef WG       Group_t;
    typedef Instance Instance_t;

    WG       *group;
    Instance  instance;

    Winstance(WG *group_=0) : group(group_) {}
    Winstance(Instance inst, WG *group_=0)
    : group(group_), instance(inst) {}
    Winstance(const Winstance &that)
    : group(that.group), instance(that.instance) {}

    Winstance &operator = (const Winstance &that)
    { group=that.group; instance=that.instance; }
  };

  template <class Group>
  class Winstance<Group,void>
  {
  public:
    typedef Group Group_t;
    typedef void  CommonBase_t;

    Group *group;

    Winstance() : group(0) {}
    Winstance(const Winstance &that) : group(that.group) {}

    Winstance &operator = (const Winstance &that) { group=that.group; }
  };

  //template <class W, class WG, class WCommon, class WGCommon, class Instance>//
 // WFactory                                                                  //
//                                                                           //

  template <class W, class WG, class WCommon, class WGCommon, class Instance>
  class WFactory : public FactoryBase<Weight,Instance>
  {
  public:
    typedef typename W::WBase_t                                WBase_t;
    typedef typename WG::WGBase_t                              WGBase_t;
    typedef W                                                  Item_t;
    typedef WG                                                 Group_t;
    typedef WCommon                                            Common_t;
    typedef WGCommon                                           WGCommon_t;
    typedef Winstance<WGBase_t,Instance>                      *Instance_t;
    typedef Instance                                           WGInstance_t;
    typedef Factory<Item_t,Common_t,Instance_t,WBase_t>        WFactory_t;
    typedef Factory<Group_t,WGCommon_t,WGInstance_t,WGBase_t>  WGFactory_t;

  private:
    WFactory();
    WFactory(const WFactory &that);

  protected:
    WGBase_t    *wg;
    WFactory_t  *wf;
    WGFactory_t *wgf;

  public:
    WFactory(WFactory_t &wf_, WGFactory_t &wgf_, WGBase_t *wg_=0)
    : wf(&wf_), wgf(&wgf_), wg(wg_) {}

    WGBase_t *get_wg() const { return wg; }
    virtual Weight *create(WGInstance_t wg_inst)
    {
      if (wg && !wfactory_can_handle(*wg,wg_inst)) wg=0;
      if (!wg) wg=wgf->create(wg_inst);
      WInstance_t w_inst(wg_inst,wg);
      WBase_t *w=wf->create(&w_inst);
      wfactory_attach(*wg,*w,wg_inst);
      return w;
    }
  };

  template <class W, class WG, class WCommon, class WGCommon>
  class WFactory<W,WG,WCommon,WGCommon,void>
  : public FactoryBase<Weight,void>
  {
  public:
    typedef typename W::WBase_t                        WBase_t;
    typedef typename WG::WGBase_t                      WGBase_t;
    typedef W                                          Item_t;
    typedef WG                                         Group_t;
    typedef WCommon                                    Common_t;
    typedef WGCommon                                   WGCommon_t;
    typedef WGBase_t                                  *Instance_t;
    typedef void                                       WGInstance_t;
    typedef Factory<Item_t,Common_t,void,WBase_t>      WFactory_t;
    typedef Factory<Group_t,WGCommon_t,void,WGBase_t>  WGFactory_t;

  private:
    WFactory();
    WFactory(const WFactory &that);

  protected:
    WGBase_t    *wg;
    WFactory_t  *wf;
    WGFactory_t *wgf;

  public:
    WFactory(WFactory_t &wf_, WGFactory_t &wgf_, WGBase_t *wg_=0)
    : wf(&wf_), wgf(&wgf_), wg(wg_) {}

    WGBase_t *get_wg() const { return wg; }
    virtual Weight *create()
    {
      if (wg && !wfactory_can_handle(*wg)) wg=0;
      if (!wg) wg=wgf->create();
      WBase_t *w=wf->create(wg);
      wfactory_attach(*wg,*w);
      return w;
    }
    
  };

}

#endif /* __ANN_WFACTORY_H */
