/***************************************************************************
                          ann_wgroup_set.h  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_WGROUP_SET_H
#define __ANN_WGROUP_SET_H

#include <ctype.h>
#include <set>
#include <iostream>

#include <cmp_ptr.h>
#include <ann_weight.h>
#include <ann_log.h>

namespace ANN
{

  class WGScommon;
  template<class W, class Instance=typename W::Instance_t>
  class Wgroup_set;

  //                 //
 // class WGScommon //
//                 //

  class WGScommon
  {
  private:
    WGScommon();

  public:
    Log *log;

    WGScommon(Log &log_) : log(&log_) {}
    WGScommon(const WGScommon &that) : log(that.log) {}

    WGScommon &operator = (const WGScommon &that)
    { log=that.log; return *this; }
  };
  
  //template<class W, class Instance>//
 // class Wgroup_set                //
//                                 //

  template <class W, class Instance>
  class Wgroup_set : public virtual Wgroup
  {
  public:
    typedef W                          WBase_t;
    typedef Log                        Common_t;
    typedef Instance                   Instance_t;
    typedef std::set<WBase_t *,CmpPtr> Wset;
    typedef typename Wset::iterator    Witerator;

  private:
    Wgroup_set(const Wgroup_set &that);

  protected:
    Wset  w_set;
    Log  *log;

  public:
    Wgroup_set(Log &log_) : log(&log_) {}
    Wgroup_set(Log &log_, Instance_t inst) : log(log_) {}
    Wgroup_set(WGScommon &c, Instance_t inst) : log(c.log) {}
    virtual ~Wgroup_set() {}

    virtual bool can_handle(Instance_t inst) { return true; }
    virtual void attach(WBase_t &w, Instance_t inst) { w_set.insert(&w); }
    virtual bool detach(WBase_t &w)
    {
      w_set.erase(&w);
      if (!w_set.empty()) return true;
      delete this;
      return false;
    }
    virtual bool has_label(const char *label);
    void         store_weights();
    virtual void updated(Trainer &trainer) { store_weights(); }
  };

  template <class W>
  class Wgroup_set<W,void> : public virtual Wgroup
  {
  public:
    typedef W                           WBase_t;
    typedef Log                         Common_t;
    typedef void                        Instance_t;
    typedef std::set<WBase_t *, CmpPtr> Wset;
    typedef typename Wset::iterator     Witerator;

  private:
    Wgroup_set(const Wgroup_set &that);

  protected:
    Wset  w_set;
    Log  *log;

  public:
    Wgroup_set(Log &log_) : log(&log_) {}
    Wgroup_set(WGScommon &c) : log(c.log) {}
    virtual ~Wgroup_set() {}

    virtual bool can_handle() { return true; }
    virtual void attach(WBase_t &w) { w_set.insert(&w); }
    virtual bool detach(WBase_t &w)
    {
      w_set.erase(&w);
      if (!w_set.empty()) return true;
      delete this;
      return false;
    }
    virtual bool has_label(const char *label);
    void         store_weights();
    virtual void updated(Trainer &trainer) { store_weights(); }
  };

} // namespace ANN

template <class W, class Instance>
bool
ANN::Wgroup_set<W,Instance>::has_label(const char *label_)
{
  Witerator it=w_set.begin();
  Witerator end=w_set.end();
  while (it!=end) if ((*it++)->has_label(label_)) return true;
  return false;
}

template <class W, class Instance>
void
ANN::Wgroup_set<W,Instance>::store_weights()
{
  if (log->is_active())
  {
    Witerator it=w_set.begin();
    Witerator end=w_set.end();
    while (it!=end) (*it++)->store(*log);
  }
}

template <class W>
bool
ANN::Wgroup_set<W,void>::has_label(const char *label_)
{
  Witerator it=w_set.begin();
  Witerator end=w_set.end();
  while (it!=end) if ((*it++)->has_label(label_)) return true;
  return false;
}

template <class W>
void
ANN::Wgroup_set<W,void>::store_weights()
{
  if (log->is_active())
  {
    Witerator it=w_set.begin();
    Witerator end=w_set.end();
    while (it!=end) (*it++)->store(*log);
  }  
}

#endif /* __ANN_WGROUP_SET_H */
