/***************************************************************************
                          ann_wmember.h  -  description
                             -------------------
    begin                : wto kwi 22 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __ANN_WMEMBER_H
#define __ANN_WMEMBER_H

#include <ann_weight.h>
#include <ann_wcommon.h>

namespace ANN
{

  template <class W, class WG=typename W::WGBase_t,
            class Common=Wcommon<WG,typename W::Instance_t>,
            class Instance=typename W::Instance_t>
  class Wmember;

  //template <class W, class WG, class Common, class Instance>//
 // class Wmember                                            //
//                                                          //

  template <class W, class WG, class Common, class Instance>
  class Wmember : public W
  {
  public:
    typedef typename W::Base_t       Base_t;
    typedef W                        Item_t;
    typedef WG                       WGBase_t;
    typedef Common                   Common_t;
    typedef Instance                 Instance_t;
    typedef FactoryBase<WG,Instance> WGFactory_t;
    
  private:
    WG *group;

    void detach_group() { if (group) group->detach(*this); }
    void attach_group(WG &group_, Instance inst)
    { group_.attach(*this,inst); group=&group_; }

  public:
    Wmember(Common &c, Instance inst) : W(c,inst)
    {
      if (c.wg && !c.wg->can_handle(inst)) c.wg=0;
      if (!c.wg) c.wg=c.wgf->create(inst);
      attach_group(*c.wg,inst);
    }
    virtual ~Wmember() { detach_group(); }

    virtual Wgroup *get_group() const { return group; }
    virtual WG     &the_group() const { return *group; }
    virtual void    set_group(WG &group_, Instance inst)
    { detach_group(); attach_group(group_,inst); }
  };

  template <class W, class WG, class Common>
  class Wmember<W,WG,Common,void> : public W
  {
  public:
    typedef typename WG::WBase_t Base_t;
    typedef W                    Item_t;
    typedef WG                   WGBase_t;
    typedef Common               Common_t;
    typedef void                 Instance_t;
    typedef FactoryBase<WG,void> WGFactory_t;
    typedef Factory<W>           Factory_t;

  private:
    WG *group;

    void detach_group() { if (group) group->detach(*me); }
    void attach_group(WG &group_) { group_.attach(*me); group=&group_; }

  public:
    Wmember(Common &c) : W(c)
    {
      if (c.wg && !c.wg->can_handle()) c.wg=0;
      if (!c.wg) c.wg=c.wgf->create();
      attach_group(*c.wg);
    }
    virtual ~Wmember() { detach_group(); }

    virtual Wgroup *get_group() const { return group; }
    virtual WG     &the_group() const { return *group; }
    virtual void    set_group(WG &group_)
    { detach_group(); attach_group(group_); }
  };

} // namespace ANN

#endif /* __ANN_WMEMBER_H */
