/***************************************************************************
                          ann_grad_weight.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <ann_grad_weight.h>

  //                           //
 // class ANN::GradWeight_Grp //
//                           //

double
ANN::GradWeight_Grp::get_random()
{
  return (2.0*rand())/RAND_MAX-1;
}

  //                  //
 // class GradWeight //
//                  //

ANN::GradWeight::~GradWeight()
{
}

void
ANN::GradWeight::prepare(Trainer &t)
{
  size_t i, l=d.get_size();
  for (i=0; i<l; ++i) d.get_data()[i]=0;
}

void
ANN::GradWeight::adapt(const Term &d_)
{
  size_t i, l=d.get_size();
  for (i=0; i<l; ++i) d.get_data()[i]+=d_.get_data()[i];
}

void
ANN::GradWeight::update(Trainer &t)
{
  size_t i, l=w.get_size();
  double etha=the_group().etha;
  for (i=0; i<l; ++i)
  {
    w.get_data()[i]-=etha*d.get_data()[i];
    d.get_data()[i]=0;
  }
}

  //                               //
 // class ANN::MomentumWeight_Grp //
//                               //

double
ANN::MomentumWeight_Grp::get_random()
{
  return (2.0*rand())/RAND_MAX-1;
}

  //                      //
 // class MomentumWeight //
//                      //

ANN::MomentumWeight::~MomentumWeight()
{
}

void
ANN::MomentumWeight::reset(Trainer &t)
{
  size_t i, l=delta.get_size();
  for (i=0; i<l; ++i) delta.get_data()[i]=0;
}

void
ANN::MomentumWeight::prepare(Trainer &t)
{
  size_t i, l=d.get_size();
  for (i=0; i<l; ++i) d.get_data()[i]=0;
}

void
ANN::MomentumWeight::adapt(const Term &d_)
{
  size_t i, l=d.get_size();
  for (i=0; i<l; ++i) d.get_data()[i]+=d_.get_data()[i];
}

void
ANN::MomentumWeight::update(Trainer &t)
{
  size_t i, l=w.get_size();
  double etha=the_group().etha;
  double alpha=the_group().alpha;
  for (i=0; i<l; ++i)
  {
    delta.get_data()[i]*=alpha;
    delta.get_data()[i]-=etha*d.get_data()[i];
    w.get_data()[i]+=delta.get_data()[i];
    d.get_data()[i]=0;
  }
}

