/***************************************************************************
                          ann_log.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include <ann_log.h>
#include <ann_lstore_set.h>

  //                //
 // class ANN::Log //
//                //

ANN::Log::~Log()
{
  if (children) delete children;
}

void
ANN::Log::log_term(const Term &t, const char *label)
{
  size_t i, l=t.get_size(), d=t.get_sizes().dim();
  char   sub_beg=get_sublevel_beg();
  char   sub_end=get_sublevel_end();
  nl(0,label);
  if (!l || (d==1) || (!sub_beg && !sub_end))
  {
    if (sub_beg) log_chr(sub_beg);
    for (i=0; i<l; ++i) log_double(t[i]);
    if (sub_end) log_chr(sub_end);
  }
  else
  {
    Loc    incr(t.get_sizes()), lim(d);
    size_t j;
    i=1;
    for (j=d-1; j>0; --j) i=(incr[j]*=i);
    i=0;
    while (i<l)
    {
      for (j=d-1; (j>0) && (i==lim[j]); --j)
      {
        lim[j]+=incr[j];
        nl(0,0);
        if (sub_beg) log_chr(sub_beg);
        nl(1,0);
      }
      log_double(t[i]);
      ++i;
      for (j=d-1; (j>0) && (i==lim[j]); --j)
      {
        nl(-1,0);
        if (sub_end) log_chr(sub_end);
        nl(0,0);
      }  
    }
  }
  nl(0,0);
}

bool
ANN::Log::attach(LogRec &child)
{
  if (!children) children=new Lstore_set;
  return children->attach(child);
}

bool
ANN::Log::detach(LogRec &child)
{
  return children && children->detach(child);
}

bool
ANN::Log::is_active() const
{
  return true;
}

void
ANN::Log::reset()
{
  if (children) children->reset();
  nl(-get_margin(),"Reseting log");
  nl(0,0);
}

void
ANN::Log::prepare(size_t epoch, bool active_)
{
  if (children) children->prepare(epoch);
  if (active_)
  {
    nl(0,"epoch start:");
    log_uint(epoch);
    nl(2,0);
  }  
}

void
ANN::Log::stepped(size_t num, double error, bool active_)
{
  if (children) children->stepped(num,error);
  if (active_)
  {
    nl(-1,"step finished:");
    log_uint(num);
    log_str("partial error:");
    log_double(error);
    nl(1,0);
  }  
}

void
ANN::Log::update(size_t epoch, double error, bool active_)
{
  if (children) children->update(epoch,error);
  if (active_)
  {
    nl(-2,"epoch finished:");
    log_uint(epoch);
    log_str("error:");
    log_double(error);
    nl(0,0);
  }  
}

  //                   //
 // class ANN::LogRec //
//                   //

ANN::LogRec::~LogRec()
{
  orig->detach(*this);
}

void
ANN::LogRec::new_line(int ch_marg, const char *label)
{
  if (active) orig->nl(ch_marg,label);
}

void
ANN::LogRec::log_chr(char chr)
{
  if (active) orig->log(chr);
}

void
ANN::LogRec::log_str(const char *str)
{
  if (active) orig->log(str);
}

void
ANN::LogRec::log_int(int i)
{
  if (active) orig->log(i);
}

void
ANN::LogRec::log_uint(unsigned int u)
{
  if (active) orig->log(u);
}

void
ANN::LogRec::log_double(double d)
{
  if (active) orig->log(d);
}

void
ANN::LogRec::log_term(const Term &t, const char *label)
{
  if (active) orig->log(t,label);
}

void
ANN::LogRec::prop_reset()
{
  if (children) children->reset();
}

void
ANN::LogRec::prop_prepare(size_t epoch)
{
  if (children) children->prepare(epoch);
}

void
ANN::LogRec::prop_stepped(size_t num, double error)
{
  if (children) children->stepped(num,error);
}

void
ANN::LogRec::prop_update(size_t epoch, double error)
{
  if (children) children->update(epoch,error);
}

ANN::Clock &
ANN::LogRec::get_clock() const
{
  return orig->get_clock();
}

char
ANN::LogRec::get_separator() const
{
  return orig->get_separator();
}

char
ANN::LogRec::get_sublevel_beg() const
{
  return orig->get_sublevel_beg();
}

char
ANN::LogRec::get_sublevel_end() const
{
  return orig->get_sublevel_end();
}

size_t
ANN::LogRec::get_margin() const
{
  return orig->get_margin(); 
}

bool
ANN::LogRec::is_active() const
{
  return active;
}

void
ANN::LogRec::reset()
{
  active=false;
  orig->reset();
}

void
ANN::LogRec::prepare(size_t epoch, bool active_)
{
  orig->prepare(epoch,active && active_);
}

void
ANN::LogRec::stepped(size_t num, double error, bool active_)
{
  orig->stepped(num,error,active && active_);
}

void
ANN::LogRec::update(size_t epoch, double error, bool active_)
{
  orig->update(epoch,error,active && active_);
}

ANN::Log &
ANN::LogRec::flush()
{
  orig->flush();
  return *this;
}

  //                   //
 // class ANN::Lstore //
//                   //

ANN::Lstore::~Lstore()
{  
}
