/***************************************************************************
                          ann_log_io.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <ann_log_io.h>
#include <time.h>
#include <string.h>

  //                  //
 // class ANN::LogIO //
//                  //

ANN::LogIO::LogIO(std::ostream &os_, const char sep_, char sub_beg_,
                  char sub_end_)
: os(&os_), marg(0), clock(&Clock::clock), sep(sep_ ? sep_ : ' '),
  sub_beg(sub_beg_), sub_end(sub_end_), is_first(true)
{
}

ANN::LogIO::LogIO(Clock &clock_, std::ostream &os_, char sep_, char sub_beg_,
                  char sub_end_)
: os(&os_), marg(0), clock(&clock_), sep(sep_ ? sep_ : ' '),
  sub_beg(sub_beg_), sub_end(sub_end_), is_first(true)
{
}

ANN::LogIO::~LogIO()
{
  if (!is_first) *os << std::endl;
  *os << std::flush;
}

void
ANN::LogIO::write_sep()
{
  if (is_first)
  {
    clock->stamp(stamp);
    *os << stamp.c_str() << '\t';
    for (int i=0; i<marg; ++i) *os << sep << sep;
    is_first=false;
  }
  else *os << sep;
}

void
ANN::LogIO::new_line(int ch_marg, const char *label)
{
  if (!is_first) *os << std::endl;
  marg+=ch_marg;
  is_first=true;
  if (label) log_str(label);
}

void
ANN::LogIO::log_chr(char chr)
{
  write_sep();
  if (chr) *os << chr;
}

void
ANN::LogIO::log_str(const char *str)
{
  write_sep();
  if (str) *os << str;
}

void
ANN::LogIO::log_int(int i)
{
  write_sep();
  *os << i;
}

void
ANN::LogIO::log_uint(unsigned int u)
{
  write_sep();
  *os << u;
}

void
ANN::LogIO::log_double(double d)
{
  write_sep();
  *os << d;
}

ANN::Clock &
ANN::LogIO::get_clock() const
{
  return *clock;
}

char
ANN::LogIO::get_separator() const
{
  return sep;
}

char
ANN::LogIO::get_sublevel_beg() const
{
  return sub_beg;
}

char
ANN::LogIO::get_sublevel_end() const
{
  return sub_end;
}

size_t
ANN::LogIO::get_margin() const
{
  return marg;
}

ANN::Log &
ANN::LogIO::flush()
{
  os->flush();
  return *this;
}

