/***************************************************************************
                          ann_multilayer.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>

#include <ann_multilayer.h>

  //                       //
 // class ANN::Multilayer //
//                       //

ANN::Multilayer::LayerData::LayerData(Element &elem_, Element *former)
: elem(&elem_), in(former ? new Term(elem_.get_in_sizes()) : 0),
  err(former && former->get_evaluator() ? new Term(elem_.get_in_sizes()) : 0)
{
}

ANN::Multilayer::~Multilayer()
{
  L_it it=layers.begin();
  L_it end=layers.end();
  while (it!=end) delete *it++;
}

bool
ANN::Multilayer::add_layer(Element *elem_)
{
  if (!elem_) return false;
  size_t s=(layers.size() ? layers.back()->elem->get_out_size()
                          : in_size.total_size());
  if (!s || (s!=elem_->get_in_size())) return false;
  layers.push_back(new LayerData(*elem_,layers.size() ? layers.back()->elem
                                                      : 0));
  return true;
}

bool
ANN::Multilayer::add_layer(NeuralFactoryBase *factory)
{
  if (!factory) return false;
  Init inst(layers.size() ? layers.back()->elem->get_out_sizes() : in_size,
            label);
  if (!inst.size->total_size()) return false;  
  inst.set_label("layer",inst.length());
  inst.set_index(layers.size(),inst.length());
  Element *elem_=factory->create(&inst);
  layers.push_back(new LayerData(*elem_,layers.size() ? layers.back()->elem
                                                      : 0));
  return true;
}

const ANN::Size &
ANN::Multilayer::get_out_sizes() const
{
  return layers.size() ? layers.back()->elem->get_out_sizes() : in_size;
}

const ANN::Evaluator *
ANN::Multilayer::get_evaluator() const
{
  return layers.size() ? layers.back()->elem->get_evaluator() : 0;
}

void
ANN::Multilayer::calc(const Term &in, Term &out)
{
  L_it        it=layers.begin();
  L_it        end=layers.end();
  const Term *in_=&in;
  Term       *out_;
  LayerData  *l;
  while (it!=end)
  {
    l=*it++;
    l->elem->calc(*in_, *(out_=(it==end ? &out : (*it)->in)));
    in_=out_;
  }
}

void
ANN::Multilayer::reset(Trainer &t, bool b_reload)
{
  L_it it=layers.begin();
  L_it end=layers.end();
  while (it!=end) (*it++)->elem->reset(t,b_reload);
}

void
ANN::Multilayer::prepare(Trainer &t)
{
  L_it it=layers.begin();
  L_it end=layers.end();
  while (it!=end) (*it++)->elem->prepare(t);
}

void
ANN::Multilayer::update(Trainer &t)
{
  L_it it=layers.begin();
  L_it end=layers.end();
  while (it!=end) (*it++)->elem->update(t);
}

void
ANN::Multilayer::adapt(const Term &in, const Term &out,
                       const Term *out_err, Term *in_err)
{
  L_rit       it=layers.rbegin();
  L_rit       end=layers.rend();
  const Term *in_, *out_=&out, *out_err_=out_err;
  Term       *in_err_;
  LayerData *l;
  while (it!=end)
  {
    l=*it++;
    l->elem->adapt(*(in_=(it==end ? &in : l->in)), *out_,
                   out_err_, in_err_=(it==end ? in_err : l->err));
    out_=in_;
    out_err_=in_err_;
  }
}

