/***************************************************************************
                          ann_perceptron.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <ann_perceptron.h>

  //                             //
 // class Perceptron::Sigmoidal //
//                             //

double
ANN::Perceptron::Sigmoidal::fun(double x) const
{
  if (x<0)
  {
    x=exp(x);
    return x/(1+x);
  }
  else
  {
    x=exp(-x);
    return 1/(1+x);
  }
}

double
ANN::Perceptron::Sigmoidal::deriv(double x, double y) const
{
  return y*(1-y);
}

  //                          //
 // class Perceptron::Linear //
//                          //

double
ANN::Perceptron::Linear::fun(double x) const
{
  return x;
}

double
ANN::Perceptron::Linear::deriv(double x, double y) const
{
  return 1;
}

  //                             //
 // class Perceptron::Evaluator //
//                             //
#include <iostream>

double
ANN::Perceptron::Evaluator::evaluate(const Term &in, const Term &desired,
                                     const Term &out, Term &err) const
{
  size_t i, l=out.get_size();
  double d, e=0;
  for (i=0; i<l; ++i)
  {
    std::cout << out[i] << " " << desired[i] << std::endl;
    err[i]=d=out[i]-desired[i];
    e+=d*d;
  }
  return e/2;
}

  //                  //
 // class Perceptron //
//                  //

ANN::Perceptron::Evaluator ANN::Perceptron::evaluator;
ANN::Perceptron::Sigmoidal ANN::Perceptron::sigmoidal;
ANN::Perceptron::Linear    ANN::Perceptron::linear;

ANN::Perceptron::Perceptron(const Param &param, const Init *inst)
: Element(inst), out_size(param.out_size), bias(param.bias),
  w(param.out_size), act(param.act), sum(*inst->size)
{
  size_t in_len=inst->size->total_size()+bias;
  size_t out_len=param.out_size.total_size();
  Size w_size(1,&in_len);
  Init sub_inst(w_size,label);
  sub_inst.set_label("neuron",sub_inst.length());
  size_t pos=sub_inst.length(), i;
  for (i=0; i<out_len; ++i)
  {
    sub_inst.set_index(i,pos);
    sub_inst.set_label("weights",sub_inst.length());
    w[i]=param.wf->create(&sub_inst);
  }
}

ANN::Perceptron::~Perceptron()
{
}

const ANN::Size &
ANN::Perceptron::get_out_sizes() const
{
  return out_size;
}

const ANN::Evaluator *
ANN::Perceptron::get_evaluator() const
{
  return &evaluator;
}

void
ANN::Perceptron::reset(Trainer &t, bool b_reload)
{
  if (b_reload)
  {
    size_t i, n=out_size.total_size();
    for (i=0; i<n; ++i) t.register_weight(*w[i]);
  }
}

void
ANN::Perceptron::calc(const Term &in, Term &out)
{
  size_t        i, n=out_size.total_size();
  size_t        j, m=in_size.total_size();
  const double *x=in.get_data(), *u;
  double       *X=sum.get_data();
  for (i=0; i<n; ++i)
  {
    u=w[i]->values().get_data();
    X[i]=(bias ? u[m] : 0);
    for (j=0; j<m; ++j) X[i]+=x[j]*u[j];
    out[i]=(*act)(X[i]);
  }
}

void
ANN::Perceptron::adapt(const Term &in, const Term &out,
                       const Term *out_err, Term *in_err)
{
  size_t        i, n=out_size.total_size();
  size_t        j, m=in_size.total_size()+bias;
  Term          grad(1,&m);
  const double *u, *x=in.get_data(), *y=out.get_data(), *e=out_err->get_data();
  double       *X=sum.get_data(), *d=grad.get_data(), E, *e_;  
  if (bias) --m;
  if (in_err)
  {
    e_=in_err->get_data();
    for (j=0; j<m; ++j) e_[j]=0;
  }  
  for (i=0; i<n; ++i)
  {
    E=e[i]*(*act)(X[i],y[i]);
    u=w[i]->values().get_data();
    if (bias) d[m]=E;
    for (j=0; j<m; ++j)
    {
      if (in_err) e_[j]+=E*u[j];
      d[j]=E*x[j];
    }  
    w[i]->adapt(grad);
  }
}

