/***************************************************************************
                          ann_trainer.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <ann_trainer.h>
#include <ann_wstore_set.h>

  //                    //
 // class ANN::Trainer //
//                    //

ANN::Trainer::Trainer(Log *log_, Wstore *ws_)
: ws(ws_), log(log_), ds(0), elem(0)
{
  if (!ws) ws=new Wstore_set;
}

ANN::Trainer::Trainer(Element &elem_, Log *log_, Wstore *ws_)
: ws(ws_), log(log_), ds(0), elem(&elem_)
{
  if (!ws) ws=new Wstore_set;
  reset(true);
}

ANN::Trainer::Trainer(DS &ds_, Element &elem_, Log *log_, Wstore *ws_)
: ws(ws_), log(log_), ds(&ds_), elem(&elem_)
{
  if (!ws) ws=new Wstore_set;
  reset(true);
}

ANN::Trainer::Trainer(DS &ds_, Log *log_, Wstore *ws_)
: ws(ws_), log(log_), ds(&ds_), elem(0)
{
  if (!ws) ws=new Wstore_set;
}

ANN::Trainer::~Trainer()
{
  if (ws) delete ws;
}

bool
ANN::Trainer::matching(const DS &ds_, const Element &elem_) const
{
  const Evaluator *e;
  if (!elem_.get_in_size()) return false;
  if (e=elem_.get_evaluator())
  {
    const DS *out=ds_.des_output();
    if (!e->trivial_match())
    {
      static Size zero;
      if (!e->matching(elem_.get_out_sizes(),(out ? out->get_sizes() : zero)))
        return false;
    }
    else if (!out || !elem_.get_out_size()
             || (elem_.get_out_size()!=out->get_size()))
      return false;

  }
  return true;
}

void
ANN::Trainer::prepare()
{
  if (log) log->prepare(epoch);
  elem->prepare(*this);
  ws->prepare(*this);
}

void
ANN::Trainer::update()
{
  ws->update(*this);
  elem->update(*this);
  if (log) log->update(epoch, error);
}

bool
ANN::Trainer::act_on(DS &ds_)
{
  if (elem && !matching(ds_,*elem)) return false;
  ds=ds_.input();
  return true;
}

bool
ANN::Trainer::act_on(Element &elem_)
{
  if (ds && !matching(*ds,elem_)) return false;  
  elem=&elem_;
  reset(true);
  return true;
}

void
ANN::Trainer::reset(bool b_reload)
{
  epoch=0;
  if (b_reload) ws->erase_weights();
  if (log) log->reset();
  if (elem) elem->reset(*this,b_reload);
  ws->reset(*this);
}

void
ANN::Trainer::randomize()
{
  reset();
  ws->randomize();
}

void
ANN::Trainer::register_weight(Weight &w)
{
  ws->register_weight(w);
}
    
bool
ANN::Trainer::load(Parser_Log &parser, time_t begin, time_t end)
{
  reset();
  return ws->load(parser,begin,end);
}

void
ANN::Trainer::store(Log &log_)
{
  ws->store(log_);  
}

size_t
ANN::Trainer::train(size_t epochs, double stop_error, size_t chk_last)
{
  if (!ds || !elem) return 0;
  const Evaluator *e=elem->get_evaluator();
  size_t           epoch_=0, last, i, l;
  double          *last_errors, acc_error;
  Term             t_out(elem->get_out_sizes());
  Term             t_err;
  if (e) t_err.resize(elem->get_out_sizes());
  if (chk_last) --chk_last;
  if (chk_last)
  {
    last_errors=new double[chk_last];
    last=chk_last;
    do last_errors[--last]=0; while (last);
    acc_error=stop_error;
  }
  ds->reset();
  l=ds->get_len();
  while (epoch_<epochs)
  {
    error=0;
    prepare();
    i=0;
    for (;;)
    {
      elem->calc(ds->term(), t_out);
      if (e) error+=e->evaluate_ds(*ds, t_out, t_err);
      elem->adapt(ds->term(), t_out, e ? &t_err : 0, 0);
      if (log) log->stepped(i,error);
      if (++i>=l) break;
      ds->next();
    }
    update();
    ds->next();
    ++epoch;
    ++epoch_;
    if (chk_last)
    {
      if (last==chk_last)
      {
        acc_error=error;
        do acc_error+=last_errors[--last]; while (last);
      }
      else
      {
        acc_error+=error-last_errors[last];
        last_errors[last]=error;
        ++last;
      }
      if (acc_error<stop_error) break;
    }
    else if (error<stop_error) break;
  }
  if (chk_last) delete last_errors;
  return epoch_;
}
