/***************************************************************************
                          ann_weight.cpp  -  description
                             -------------------
    begin                : pon kwi 14 2003
    copyright            : (C) 2003 by Bartosz Lis
    email                : bartoszl@ics.p.lodz.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <string.h>

#include <ann_weight.h>
#include <ann_tc_set.h>

  //                   //
 // class ANN::Weight //
//                   //

ANN::Weight::~Weight()
{
  if (label) delete label;
}

bool
ANN::Weight::has_label(const char *label_)
{
  return !strcmp(label_,label);
}

ANN::Wgroup *
ANN::Weight::get_group() const
{
  return 0;
}

void
ANN::Weight::reset(Trainer &t)
{
}

void
ANN::Weight::prepare(Trainer &t)
{
}

void
ANN::Weight::randomize()
{
  Wgroup *g=get_group();
  size_t i;
  for (i=0; i<w.get_size(); ++i)
    w[i]=(g ? g->get_random() : Wgroup::def_random());
}

bool
ANN::Weight::load(Parser_Log &parser, time_t begin, time_t end)
{
  if ((begin!=(time_t)-1) && !parser.seek(begin)) return false;
  if (!parser.find(label)) return false;
  if ((end!=(time_t)-1) && (end<parser.when())) return false;
  TC_set tc;
  if (!parser.load(tc)) return false;
  if ((end!=(time_t)-1) && (end<parser.when())) return false;
  if (tc.get_sizes().total_size()!=w.get_size()) return false;
  tc.get_next(w);
  return true;
}

void
ANN::Weight::store(Log &log)
{
  log.log(w,label);
}

  //                   //
 // class ANN::Wgroup //
//                   //

ANN::Wgroup::~Wgroup()
{
}

#define WGROUP_SIGNATURE "wieght_group["

bool
ANN::Wgroup::signature_find(Parser_Log &parser, time_t begin, time_t end)
{
  char   *tail, sep=parser.get_separator();
  size_t  l;
  bool    cont=true, ret;
  if ((begin!=(time_t)-1) && !parser.seek(begin)) return false;
  while (cont) 
  {
    tail=0;
    if (!parser.find(WGROUP_SIGNATURE,tail)) return false;
    if (!tail) return false;
    if ((end!=(time_t)-1) && (end<parser.when())) ret=cont=false;
    else if (l=strlen(tail)) if (tail[--l]==']') while (l) if (tail[--l]!=sep)
    {
      tail[++l]=0;
      cont=!(ret=has_label(tail));
      break;
    }
    delete tail;
  }
  return ret;
}

void
ANN::Wgroup::signature_write(Log &log, Weight &w)
{
  log.nl(0,WGROUP_SIGNATURE) << w.get_label() << ']';
}

void
ANN::Wgroup::reset(Trainer &trainer)
{
}

void
ANN::Wgroup::prepare(Trainer &trainer)
{
}

void
ANN::Wgroup::prepared(Trainer &trainer)
{
}

void
ANN::Wgroup::update(Trainer &trainer)
{
}

void
ANN::Wgroup::updated(Trainer &trainer)
{
}

double
ANN::Wgroup::get_random()
{
  return def_random();
}

double
ANN::Wgroup::def_random()
{
  return (2.0*rand())/RAND_MAX-1;
}

bool
ANN::Wgroup::load(Parser_Log &parser, time_t begin, time_t end)
{
  return true;
}

void
ANN::Wgroup::store(Log &log)
{
}

  //                   //
 // class ANN::Wstore //
//                   //

ANN::Wstore::~Wstore()
{
}

